#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "lr.h"

#include "mex.h"

#define CMD_LEN 2048
#define Malloc(type,n) (type *)malloc((n)*sizeof(type))

void exit_with_help()
{
	mexPrintf(
	"Usage: model = lrtrain(training_label_vector, training_instance_matrix, 'liblr_options');\n"
	);
}

// lr arguments
struct lr_parameter param;		// set by parse_command_line
struct lr_problem prob;		// set by read_problem
int cross_validation;
int nr_fold;

// nrhs should be 3
int parse_command_line(int nrhs, const mxArray *prhs[], char *model_file_name)
{
	int i, argc = 1;
	char cmd[CMD_LEN];
	char *argv[CMD_LEN/2];

	// default values
	param.lr_type = C_SVC;
	param.C = 1;
	param.eps = 1e-3;
	param.nr_weight = 0;
	param.weight_label = NULL;
	param.weight = NULL;
	cross_validation = 0;

	if(nrhs <= 1)
		return 1;
	if(nrhs == 2)
		return 0;

	// put options in argv[]
	mxGetString(prhs[2], cmd,  mxGetN(prhs[2]) + 1);
	if((argv[argc] = strtok(cmd, " ")) == NULL)
		return 0;
	while((argv[++argc] = strtok(NULL, " ")) != NULL)
		;

	// parse options
	for(i=1;i<argc;i++)
	{
		if(argv[i][0] != '-') break;
		if(++i>=argc)
			return 1;
		switch(argv[i-1][1])
		{
			case 's':
				param.lr_type = atoi(argv[i]);
				break;
			case 'c':
				param.C = atof(argv[i]);
				break;
			case 'e':
				param.eps = atof(argv[i]);
				break;
			case 'v':
				cross_validation = 1;
				nr_fold = atoi(argv[i]);
				if(nr_fold < 2)
				{
					mexPrintf("n-fold cross validation: n must >= 2\n");
					return 1;
				}
				break;
			case 'w':
				++param.nr_weight;
				param.weight_label = (int *)realloc(param.weight_label,sizeof(int)*param.nr_weight);
				param.weight = (double *)realloc(param.weight,sizeof(double)*param.nr_weight);
				param.weight_label[param.nr_weight-1] = atoi(&argv[i-1][2]);
				param.weight[param.nr_weight-1] = atof(argv[i]);
				break;
			default:
				mexPrintf("unknown option\n");
				return 1;
		}
	}
	return 0;
}

static void fake_answer(mxArray *plhs[])
{
	plhs[0] = mxCreateDoubleMatrix(0, 0, mxREAL);
}

// Interface function of matlab
// now assume prhs[0]: label prhs[1]: features
void mexFunction( int nlhs, mxArray *plhs[],
		int nrhs, const mxArray *prhs[] )
{
	const char *error_msg;
	// fix random seed to have same results for each run
	// (for cross validation and probability estimation)
	srand(1);

	// Translate the input Matrix to the format such that lrtrain.exe can recognize it
	if(nrhs > 0 && nrhs < 4)
	{
		if(parse_command_line(nrhs, prhs, NULL))
		{
			exit_with_help();
			lr_destroy_param(&param);
			fake_answer(plhs);
			return;
		}

		if(mxIsSparse(prhs[1]))
		{
			prob.y = mxGetPr(prhs[0]);
			prob.l = mxGetM(prhs[1]);
			prob.n = mxGetN(prhs[1]);
			prob.x_val = mxGetPr(prhs[1]);
			prob.x_ir = (long int*) mxGetIr(prhs[1]);
			prob.x_jc = (long int*) mxGetJc(prhs[1]);
		}
		else
			mexPrintf("Training_instance_matrix must be sparse\n");

		// lrtrain's original code
		error_msg = lr_check_parameter(&prob, &param);

		if(error_msg)
		{
			if (error_msg != NULL)
				mexPrintf("Error: %s\n", error_msg);
			lr_destroy_param(&param);
			fake_answer(plhs);
			return;
		}

		if(cross_validation)
		{
			double *ptr;
			plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
			ptr = mxGetPr(plhs[0]);
			//			ptr[0] = do_cross_validation();
		}
		else
		{
			plhs[0] = mxCreateDoubleMatrix(prob.n, 1, mxREAL) ;
			double *w = mxGetPr(plhs[0]);
			lr_train(&prob, &param, w);
		}
		/*
		free(prob.x_ir);
		free(prob.x_jc);
		*/
		lr_destroy_param(&param);
	}
	else
	{
		exit_with_help();
		fake_answer(plhs);
		return;
	}
}
