function [w]=svmlin
% Matlab Interface to SVMlin
% Usage: [w,o]=svmlin(cmdline,X,Y,w)
% 
% Inputs:
%
% cmdline: a string containing command line options
%          e.g. cmdline='-A 1 -W 10.0' where the options are,  
%          -A algorithm : set algorithm (default 1)
%                         0 -- Regularized Least Squares Classification (RLSC)
%                         1 -- SVM (L2-SVM-MFN) (default choice)
%                         2 -- Multi-switch Transductive SVM (using L2-SVM-MFN)
%                         3 -- Deterministic Annealing Semi-supervised VM (using L2-SVM-MFN)
%          -W regularization parameter lambda 
%          -U regularization parameter lambda_u (see notes below)
%          -S maximum number of switches in TSVM (default 10000)
%          -R positive class fraction of unlabeled data  (default 0.5)
%          -Cp relative cost for positive examples (only available with -A 1)
%          -Cn relative cost for negative examples (only available with -A 1)
% X : data matrix (in sparse format) with examples as rows and features as columns  
% Y : label vector (same length as number of examples)
% w : (test mode) A trained weight vector is given for prediction on X.
%      In test mode, if Y is given then accuracy of prediction is printed
%      out. If Y is not available, use Y=[]
%
% Outputs: 
% w: trained weight vector
% o: (soft) predictions of w on X
%
% Example Usage:
%
% Training: 
%   For training a TSVM with maximum switching heuristic (10000
%   switches allowed per iteration), lambda=0.001, lambda_u=1 and positive
%   class ratio 0.2 (fraction of unlabeled data expected to be positive)
%           [w,o]=svmlin('-A 2 -W 0.001 -U 1 -R 0.2 -S 100000'); 
%
% Prediction: 
%           [w,o]=svmlin([],X,[],w); % the output and input w vectors are
%                                    % the same in the prediction mode. 
%                                    % The soft predictions are saved
%                                    % in the vector o.
% Evaluation: 
%           [w,o]=svmlin([],X,Y,w); % outputs accuracy too comparing the 
%                                   predictions o with the true labels Y.
%
%
% Notes: In this code, we need to take a transpose of X to convert to 
%        C compressed row format from the matlab compressed column format.
%        Addionally, a vector of ones is appended as an additional column
%        of X to incorporate the bias feature. Some checks are made to
%        ensure that the mex file runs smoothly. These modifications cause
%        this code to run slower than the C code (taking away loading time).
%        However, the time reported is the time the mex code takes which
%        should be very similar what one would get thru the C code.
%
% Author: Vikas Sindhwani (vikass@cs.uchicago.edu)
%         SVMlin (v1.0) August 2006
%
%

global y
global xt
global C
global eps

param=[1.0 1.0 1.0 10000 -1 2*C 2*C eps]';

[w,o]=svmlin_mex(param,xt,y, 2*C*ones(size(y)));
